@@ -301,8 +301,8 @@
 
     iframe.on('load', function() {
       var $state = $injector.get('$state');
-      if (response.config.attempt) {
-        // Already attempted once -- reload page
+      if (response.config.attempt > 2) {
+        // Already attempted 3 times -- reload page
         angular.element($window).off('beforeunload');
         $window.location.href = $window.ApplicationBaseURL + $state.href($state.current);
         deferred.reject();
@@ -311,12 +311,18 @@
         // Once the browser has followed the redirection, send the initial request
         $timeout(function() {
           var $http = $injector.get('$http');
-          response.config.attempt = 1;
+          if (response.config.attempt)
+            response.config.attempt++;
+          else
+            response.config.attempt = 1;
           $http(response.config).then(function(response) {
             deferred.resolve(response);
+          }, function(response) {
+            deferred.reject(response);
+          }).finally(function() {
+            $timeout(iframe.remove, 1000);
           });
-          $timeout(iframe.remove, 500);
-        }, 100); // Wait before replaying the request
+        }, 2000); // Wait before replaying the request
       }
     });
 
@@ -335,8 +341,8 @@
         // When expecting JSON but receiving HTML, assume session has expired and reload page
         var $state;
         if (response && /^application\/json/.test(response.config.headers.Accept) &&
-            /^[\n\r ]*<!DOCTYPE html/.test(response.data)) {
-          if ($window.usesCASAuthentication) {
+            /^[\n\r\t ]*<!DOCTYPE html/.test(response.data)) {
+          if ($window.usesCASAuthentication || $window.usesSAML2Authentication) {
             return renewTicket($window, $q, $timeout, $injector, response);
           }
           else {
@@ -360,8 +366,8 @@
       responseError: function(rejection) {
         var $state;
         if (/^application\/json/.test(rejection.config.headers.Accept)) {
-          // Handle CAS ticket renewal
-          if ($window.usesCASAuthentication && rejection.status == -1) {
+          // Handle SSO ticket renewal
+          if (($window.usesCASAuthentication || $window.usesSAML2Authentication) && rejection.status == -1) {
             return renewTicket($window, $q, $timeout, $injector, rejection);
           }
           else if ($window.usesSAML2Authentication && rejection.status == 401 && !$window.recovered) {
@@ -370,7 +376,7 @@
             $window.recovered = true;
             $window.location.href = $window.ApplicationBaseURL + $state.href($state.current);
           }
-          else {
+          else if (!rejection.data.quiet) {
             // Broadcast the response error
             $rootScope.$broadcast('http:Error', rejection);
           }
