@@ -6,8 +6,8 @@
   /**
    * @ngInject
    */
-  ComponentController.$inject = ['$rootScope', '$q', '$mdDialog', 'Calendar', 'Component', 'AddressBook', 'Alarm', 'Account', 'stateComponent'];
-  function ComponentController($rootScope, $q, $mdDialog, Calendar, Component, AddressBook, Alarm, Account, stateComponent) {
+  ComponentController.$inject = ['$rootScope', '$scope', '$q', '$mdDialog', 'Preferences', 'Calendar', 'Component', 'AddressBook', 'Account', 'stateComponent'];
+  function ComponentController($rootScope, $scope, $q, $mdDialog, Preferences, Calendar, Component, AddressBook, Account, stateComponent) {
     var vm = this, component;
 
     this.$onInit = function () {
@@ -70,6 +70,7 @@
                 return onCompleteDeferred.resolve(element);
               },
               locals: {
+                stateParent: $scope,
                 stateAccount: account,
                 stateMessage: message,
                 onCompletePromise: function () {
@@ -118,7 +119,7 @@
 
       c.$reply().then(function() {
         $rootScope.$emit('calendars:list');
-        Alarm.getAlarms();
+        Preferences.getAlarms();
         $mdDialog.hide();
       });
     };
@@ -204,9 +205,9 @@
   /**
    * @ngInject
    */
-  ComponentEditorController.$inject = ['$rootScope', '$scope', '$log', '$timeout', '$mdDialog', 'sgFocus', 'User', 'CalendarSettings', 'Calendar', 'Component', 'AddressBook', 'Card', 'Alarm', 'stateComponent'];
-  function ComponentEditorController($rootScope, $scope, $log, $timeout, $mdDialog, focus, User, CalendarSettings, Calendar, Component, AddressBook, Card, Alarm, stateComponent) {
-    var vm = this, component, oldStartDate, oldEndDate, oldDueDate;
+  ComponentEditorController.$inject = ['$rootScope', '$scope', '$log', '$timeout', '$window', '$element', '$mdDialog', '$mdToast', 'sgFocus', 'User', 'CalendarSettings', 'Calendar', 'Component', 'Attendees', 'AddressBook', 'Card', 'Preferences', 'stateComponent'];
+  function ComponentEditorController($rootScope, $scope, $log, $timeout, $window, $element, $mdDialog, $mdToast, focus, User, CalendarSettings, Calendar, Component, Attendees, AddressBook, Card, Preferences, stateComponent) {
+    var vm = this, component, oldStartDate, oldEndDate, oldDueDate, dayStartTime, dayEndTime;
 
     this.$onInit = function () {
       this.service = Calendar;
@@ -214,19 +215,35 @@
       this.categories = {};
       this.showRecurrenceEditor = this.component.$hasCustomRepeat;
       this.showAttendeesEditor = this.component.attendees && this.component.attendees.length;
-      //this.searchText = null;
-      this.attendeeConflictError = false;
-      this.attendeesEditor = {
-        days: getDays(),
-        hours: getHours()
-      };
 
-      if (this.component.start)
+      if (this.component.type == 'appointment') {
+        this.component.initAttendees();
+        this.attendeeConflictError = false;
+        this.attendeesEditor = {
+          days: this.component.$attendees.$days,
+          hours: getHours(),
+          containerElement: $element[0].querySelector('#freebusy')
+        };
+      }
+
+      if (this.component.start) {
         oldStartDate = new Date(this.component.start.getTime());
-      if (this.component.end)
+        this.startTime = new Date(this.component.start.getTime());
+      }
+      if (this.component.end) {
         oldEndDate = new Date(this.component.end.getTime());
-      if (this.component.due)
+        this.endTime = new Date(this.component.end.getTime());
+      }
+      if (this.component.due) {
         oldDueDate = new Date(this.component.due.getTime());
+        this.dueTime = new Date(this.component.due.getTime());
+      }
+
+      if (this.component.attendees)
+        $timeout(scrollToStart);
+
+      dayStartTime = parseInt(Preferences.defaults.SOGoDayStartTime);
+      dayEndTime = parseInt(Preferences.defaults.SOGoDayEndTime);
     };
 
     this.addAttachUrl = function () {
@@ -249,6 +266,12 @@
         this.component.repeat.month.type == 'bymonthday';
     };
 
+    this.frequencies = function () {
+      return _.filter($window.repeatFrequencies, function (frequency) {
+        return frequency[0] != 'custom' || vm.component.repeat.frequency == 'custom';
+      });
+    };
+
     this.changeFrequency = function () {
       if (this.component.repeat.frequency == 'custom')
         this.showRecurrenceEditor = true;
@@ -262,8 +285,7 @@
 
     // Autocomplete cards for attendees
     this.cardFilter = function ($query) {
-      AddressBook.$filterAll($query);
-      return AddressBook.$cards;
+      return AddressBook.$filterAll($query);
     };
 
     this.addAttendee = function (card, partial) {
@@ -280,7 +302,21 @@
             name = str.replace(new RegExp(" *<?" + email + ">? *"), '');
         vm.showAttendeesEditor |= initOrganizer;
         vm.searchText = '';
-        return new Card({ c_cn: _.trim(name, ' "'), emails: [{ value: email }] });
+        return vm.cardFilter(email).then(function (cards) {
+          if (cards.length) {
+            return cards[0];
+          } else {
+            return new Card({ c_cn: _.trim(name, ' "'), emails: [{ value: email }] });
+          }
+        }).catch(function (err) {
+          // Server error
+          return new Card({ c_cn: _.trim(name, ' "'), emails: [{ value: email }] });
+        });
+      }
+
+      function addCard(newCard) {
+        if (!vm.component.$attendees.hasAttendee(newCard))
+          vm.component.$attendees.add(newCard, options);
       }
 
       if (angular.isString(card)) {
@@ -296,29 +332,99 @@
                card.charCodeAt(i) == 44 ||   // ,
                card.charCodeAt(i) == 59) &&  // ;
               emailRE.test(address)) {
-            this.component.addAttendee(createCard(address), options);
+            createCard(address).then(addCard);
             address = '';
           }
           else {
             address += card.charAt(i);
           }
         }
-        if (address)
-          this.component.addAttendee(createCard(address), options);
+        if (address && emailRE.test(address))
+          createCard(address).then(addCard);
       }
-      else {
-        this.component.addAttendee(card, options);
+      else if (angular.isDefined(card)) {
+        if (!this.component.$attendees.hasAttendee(card))
+          this.component.$attendees.add(card, options);
         this.showAttendeesEditor |= initOrganizer;
       }
+
+      $timeout(scrollToStart);
+    };
+
+    function scrollToStart() {
+      var dayElement, scrollLeft;
+      if (!vm.attendeesEditor.containerElement) {
+        vm.attendeesEditor.containerElement = $element[0].querySelector('#freebusy');
+      }
+      dayElement = $element[0].querySelector('#freebusy_day_' + vm.component.start.getDayString());
+      if (vm.attendeesEditor.containerElement && dayElement) {
+        scrollLeft = dayElement.offsetLeft - vm.attendeesEditor.containerElement.offsetLeft;
+        vm.attendeesEditor.containerElement.scrollLeft = scrollLeft;
+      }
+    }
+
+    this.expandAttendee = function (attendee) {
+      if (attendee.members.length > 0) {
+        this.component.$attendees.remove(attendee);
+        _.forEach(attendee.members, function (member) {
+          vm.component.$attendees.add(member);
+        });
+      }
     };
 
     this.removeAttendee = function (attendee, form) {
-      this.component.deleteAttendee(attendee);
-      if (this.component.attendees.length === 0)
+      this.component.$attendees.remove(attendee);
+      if (this.component.$attendees.getLength() === 0) {
         this.showAttendeesEditor = false;
+        this.component.$attendees.remove(this.component.organizer);
+      }
       form.$setDirty();
     };
 
+    this.defaultIconForAttendee = function (attendee) {
+      if (attendee.isGroup) {
+        return 'group';
+      } else if (attendee.isResource) {
+        return 'meeting_room';
+      } else {
+        return 'person';
+      }
+    };
+
+    this.nextSlot = function () {
+      findSlot(1);
+    };
+
+    this.previousSlot = function () {
+      findSlot(-1);
+    };
+
+    function findSlot(direction) {
+      vm.adjustStartTime();
+      vm.adjustEndTime();
+      vm.component.$attendees.findSlot(direction).then(function () {
+        vm.startTime = new Date(vm.component.start.getTime());
+        vm.endTime = new Date(vm.component.end.getTime());
+      }).catch(function (err) {
+        vm.component.start = new Date(vm.component.start.getTime() + 1); // trigger update in sgFreeBusy
+        $timeout(scrollToStart);
+        $mdToast.show({
+          template: [
+            '<md-toast>',
+            '  <div class="md-toast-content">',
+            '    <md-icon class="md-warn md-hue-1">error_outline</md-icon>',
+            '    <span flex>' + err + '</span>',
+            '  </div>',
+            '</md-toast>'
+          ].join(''),
+          hideDelay: 5000,
+          position: 'top right'
+        });
+      }).finally(function () {
+        $timeout(scrollToStart);
+      });
+    }
+
     this.priorityLevel = function () {
       if (this.component && this.component.priority) {
         if (this.component.priority > 5)
@@ -331,14 +437,16 @@
     };
 
     this.changeAlarmRelation = function (form) {
-      if (this.component.type == 'task' && this.component.$hasAlarm &&
-          (this.component.start || this.component.due) &&
-          ((!this.component.start && this.component.alarm.relation == 'START') ||
-           (!this.component.due   && this.component.alarm.relation == 'END'))) {
-        form.alarmRelation.$setValidity('alarm', false);
-      }
-      else {
-        form.alarmRelation.$setValidity('alarm', true);
+      if (form.alarmRelation) {
+        if (this.component.type == 'task' && this.component.$hasAlarm &&
+            (this.component.start || this.component.due) &&
+            ((!this.component.start && this.component.alarm.relation == 'START') ||
+             (!this.component.due   && this.component.alarm.relation == 'END'))) {
+          form.alarmRelation.$setValidity('alarm', false);
+        }
+        else {
+          form.alarmRelation.$setValidity('alarm', true);
+        }
       }
     };
 
@@ -355,19 +463,21 @@
     };
 
     this.save = function (form, options) {
+      this.adjustStartTime();
+      this.adjustEndTime();
       this.changeAlarmRelation(form);
       if (form.$valid) {
         this.component.$save(options)
           .then(function(data) {
             $rootScope.$emit('calendars:list');
-            Alarm.getAlarms();
+            Preferences.getAlarms();
             $mdDialog.hide();
           }, function(response) {
             if (response.status == CalendarSettings.ConflictHTTPErrorCode &&
                 _.isObject(response.data.message))
               vm.attendeeConflictError = response.data.message;
             else
-              edit(form);
+              vm.edit(form);
           });
       }
     };
@@ -386,23 +496,11 @@
       $mdDialog.hide();
     };
 
-    function edit(form) {
-      vm.attendeeConflictError = false;
+    this.edit = function (form) {
+      this.attendeeConflictError = false;
       form.$setPristine();
       form.$setDirty();
-    }
-
-    function getDays() {
-      var days = [];
-
-      if (vm.component.start && vm.component.end)
-        days = vm.component.start.daysUpTo(vm.component.end);
-
-      return _.map(days, function(date) {
-        return { stringWithSeparator: date.stringWithSeparator(),
-                 getDayString: date.getDayString() };
-      });
-    }
+    };
 
     function getHours() {
       var hours = [];
@@ -415,10 +513,12 @@
     this.addStartDate = function (form) {
       this.component.$addStartDate();
       oldStartDate = new Date(this.component.start.getTime());
+      this.startTime = new Date(this.component.start.getTime());
       if (!this.component.due) {
         this.component.alarm.relation = 'START';
       }
       this.changeAlarmRelation(form);
+      form.$setDirty();
     };
 
     this.removeStartDate = function (form) {
@@ -427,15 +527,18 @@
         this.component.alarm.relation = 'END';
       }
       this.changeAlarmRelation(form);
+      form.$setDirty();
     };
 
     this.addDueDate = function (form) {
       this.component.$addDueDate();
       oldDueDate = new Date(this.component.due.getTime());
+      this.dueTime = new Date(this.component.due.getTime());
       if (!this.component.start) {
         this.component.alarm.relation = 'END';
       }
       this.changeAlarmRelation(form);
+      form.$setDirty();
     };
 
     this.removeDueDate = function (form) {
@@ -444,18 +547,38 @@
         this.component.alarm.relation = 'START';
       }
       this.changeAlarmRelation(form);
+      form.$setDirty();
+    };
+
+    this.adjustAllDay = function () {
+      if (!this.component.isAllDay) {
+        this.component.start.setHours(dayStartTime);
+        this.component.start.setMinutes(0);
+        this.startTime = new Date(this.component.start.getTime());
+        oldStartDate = new Date(this.component.start.getTime());
+        this.component.end.setHours(dayEndTime);
+        this.component.end.setMinutes(0);
+        this.endTime = new Date(this.component.end.getTime());
+        oldEndDate = new Date(this.component.end.getTime());
+        this.component.delta = this.component.start.minutesTo(this.component.end);
+      }
+      this.component.$attendees.updateFreeBusyCoverage();
     };
 
     this.adjustStartTime = function () {
-      if (this.component.start) {
+      var delta;
+      if (this.component.start && this.startTime) {
+        // Update the component start date
+        this.component.start.setHours(this.startTime.getHours());
+        this.component.start.setMinutes(this.startTime.getMinutes());
         // Preserve the delta between the start and end dates
-        var delta;
         delta = oldStartDate.valueOf() - this.component.start.valueOf();
         if (delta !== 0) {
           oldStartDate = new Date(this.component.start.getTime());
           if (this.component.type === 'appointment') {
             this.component.end = new Date(this.component.start.getTime());
             this.component.end.addMinutes(this.component.delta);
+            this.endTime = new Date(this.component.end.getTime());
             oldEndDate = new Date(this.component.end.getTime());
           }
           updateFreeBusy();
@@ -464,13 +587,24 @@
     };
 
     this.adjustEndTime = function () {
-      if (this.component.end) {
+      var delta;
+      if (this.component.end && this.endTime) {
+        // Update the component end date
+        this.component.end.setHours(this.endTime.getHours());
+        this.component.end.setMinutes(this.endTime.getMinutes());
         // The end date must be after the start date
-        var delta = oldEndDate.valueOf() - this.component.end.valueOf();
+        delta = oldEndDate.valueOf() - this.component.end.valueOf();
         if (delta !== 0) {
+          if (this.startTime) {
+            // Update the component start date
+            this.component.start.setHours(this.startTime.getHours());
+            this.component.start.setMinutes(this.startTime.getMinutes());
+          }
           delta = this.component.start.minutesTo(this.component.end);
-          if (delta < 0)
+          if (delta < 0) {
             this.component.end = new Date(oldEndDate.getTime());
+            this.endTime = new Date(this.component.end.getTime());
+          }
           else {
             this.component.delta = delta;
             oldEndDate = new Date(this.component.end.getTime());
@@ -481,12 +615,17 @@
     };
 
     this.adjustDueTime = function () {
-      oldDueDate = new Date(this.component.due.getTime());
+      if (this.component.due && this.dueTime) {
+        this.component.due.setHours(this.dueTime.getHours());
+        this.component.due.setMinutes(this.dueTime.getMinutes());
+        oldDueDate = new Date(this.component.due.getTime());
+      }
     };
 
     function updateFreeBusy() {
-      vm.attendeesEditor.days = getDays();
-      vm.component.updateFreeBusy();
+      vm.component.$attendees.updateFreeBusyCoverage();
+      vm.component.$attendees.updateFreeBusy();
+      $timeout(scrollToStart);
     }
   }
 
